# Generated by Django 5.1.1 on 2024-10-08 15:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_eventspipe', '0003_eventschedule'),
    ]

    operations = [
        migrations.AddField(
            model_name='eventschedule',
            name='enabled',
            field=models.BooleanField(default=False),
        ),
    ]
