# Generated by Django 5.1.1 on 2024-10-10 20:11

import django_eventspipe.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_eventspipe', '0004_eventschedule_enabled'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eventschedule',
            name='event',
            field=models.JSONField(default=dict, help_text='Initial context for a Pipeline', validators=[django_eventspipe.validators.event_validator]),
        ),
    ]
